@@ -2,47 +2,51 @@
 #
 #      Copyright (c) 1996 Malcolm Beattie
 #      Copyright (c) 2010 Reini Urban
-#      Copyright (c) 2012 cPanel Inc
+#      Copyright (c) 2012, 2013 cPanel Inc
 #
 #      You may distribute under the terms of either the GNU General Public
 #      License or the Artistic License, as specified in the README file.
 #
 package B::Stackobj;
 
-our $VERSION = '1.00_03';
+our $VERSION = '1.11';
 
 use Exporter ();
 @ISA       = qw(Exporter);
-@EXPORT_OK = qw(set_callback T_UNKNOWN T_DOUBLE T_INT VALID_UNSIGNED
-  VALID_INT VALID_DOUBLE VALID_SV REGISTER TEMPORARY);
+@EXPORT_OK = qw(set_callback T_UNKNOWN T_NUM T_INT T_STR VALID_UNSIGNED
+  VALID_INT VALID_NUM VALID_STR VALID_SV REGISTER TEMPORARY);
 %EXPORT_TAGS = (
-  types => [qw(T_UNKNOWN T_DOUBLE T_INT)],
+  types => [qw(T_UNKNOWN T_NUM T_INT T_STR)],
   flags => [
-    qw(VALID_INT VALID_DOUBLE VALID_SV
+    qw(VALID_INT VALID_NUM VALID_STR VALID_SV
       VALID_UNSIGNED REGISTER TEMPORARY)
   ]
 );
 
 use Carp qw(confess);
 use strict;
-use B qw(class SVf_IOK SVf_NOK SVf_IVisUV SVf_ROK);
+use B qw(class SVf_IOK SVf_NOK SVf_IVisUV SVf_ROK SVf_POK);
+use B::C qw(ivx nvx);
 use Config;
 
 # Types
 sub T_UNKNOWN () { 0 }
-sub T_DOUBLE ()  { 1 }
-sub T_INT ()     { 2 }
-sub T_SPECIAL () { 3 }
+sub T_INT ()     { 1 }
+sub T_NUM ()     { 2 }
+sub T_STR ()     { 3 }
+sub T_SPECIAL () { 4 }
 
 # Flags
 sub VALID_INT ()      { 0x01 }
 sub VALID_UNSIGNED () { 0x02 }
-sub VALID_DOUBLE ()   { 0x04 }
-sub VALID_SV ()       { 0x08 }
-sub REGISTER ()       { 0x10 }    # no implicit write-back when calling subs
-sub TEMPORARY ()      { 0x20 }    # no implicit write-back needed at all
-sub SAVE_INT ()       { 0x40 }    # if int part needs to be saved at all
-sub SAVE_DOUBLE ()    { 0x80 }    # if double part needs to be saved at all
+sub VALID_NUM ()      { 0x04 }
+sub VALID_STR ()      { 0x08 }
+sub VALID_SV ()       { 0x10 }
+sub REGISTER ()       { 0x20 }    # no implicit write-back when calling subs
+sub TEMPORARY ()      { 0x40 }    # no implicit write-back needed at all
+sub SAVE_INT ()       { 0x80 }    # if int part needs to be saved at all
+sub SAVE_NUM ()       { 0x100 }   # if num part needs to be saved at all
+sub SAVE_STR ()       { 0x200 }   # if str part needs to be saved at all
 
 #
 # Callback for runtime code generation
@@ -55,10 +59,14 @@ sub runtime { &$runtime_callback(@_) }
 # Methods
 #
 
+# The stack holds generally only the string ($sv->save) representation of the B object,
+# for the types sv, int, double, numeric and sometimes bool.
+# Special subclasses keep the B obj, like Const
+
 sub write_back { confess "stack object does not implement write_back" }
 
 sub invalidate {
-  shift->{flags} &= ~( VALID_INT | VALID_UNSIGNED | VALID_DOUBLE );
+  shift->{flags} &= ~( VALID_INT | VALID_UNSIGNED | VALID_NUM | VALID_STR );
 }
 
 sub invalidate_int {
@@ -66,7 +74,11 @@ sub invalidate_int {
 }
 
 sub invalidate_double {
-  shift->{flags} &= ~( VALID_DOUBLE );
+  shift->{flags} &= ~( VALID_NUM );
+}
+
+sub invalidate_str {
+  shift->{flags} &= ~( VALID_STR );
 }
 
 sub as_sv {
@@ -78,6 +90,10 @@ sub as_sv {
   return $obj->{sv};
 }
 
+sub as_obj {
+  return shift->{obj};
+}
+
 sub as_int {
   my $obj = shift;
   if ( !( $obj->{flags} & VALID_INT ) ) {
@@ -89,13 +105,22 @@ sub as_int {
 
 sub as_double {
   my $obj = shift;
-  if ( !( $obj->{flags} & VALID_DOUBLE ) ) {
+  if ( !( $obj->{flags} & VALID_NUM ) ) {
     $obj->load_double;
-    $obj->{flags} |= VALID_DOUBLE | SAVE_DOUBLE;
+    $obj->{flags} |= VALID_NUM | SAVE_NUM;
   }
   return $obj->{nv};
 }
 
+sub as_str {
+  my $obj = shift;
+  if ( !( $obj->{flags} & VALID_STR ) ) {
+    $obj->load_str;
+    $obj->{flags} |= VALID_STR | SAVE_STR;
+  }
+  return $obj->{sv};
+}
+
 sub as_numeric {
   my $obj = shift;
   return $obj->{type} == T_INT ? $obj->as_int : $obj->as_double;
@@ -106,7 +131,7 @@ sub as_bool {
   if ( $obj->{flags} & VALID_INT ) {
     return $obj->{iv};
   }
-  if ( $obj->{flags} & VALID_DOUBLE ) {
+  if ( $obj->{flags} & VALID_NUM ) {
     return $obj->{nv};
   }
   return sprintf( "(SvTRUE(%s))", $obj->as_sv );
@@ -126,14 +151,18 @@ sub peek {
   elsif ( $type == T_INT ) {
     $type = "T_INT";
   }
-  elsif ( $type == T_DOUBLE ) {
-    $type = "T_DOUBLE";
+  elsif ( $type == T_NUM ) {
+    $type = "T_NUM";
+  }
+  elsif ( $type == T_STR ) {
+    $type = "T_STR";
   }
   else {
     $type = "(illegal type $type)";
   }
   push( @flags, "VALID_INT" )    if $flags & VALID_INT;
-  push( @flags, "VALID_DOUBLE" ) if $flags & VALID_DOUBLE;
+  push( @flags, "VALID_NUM" )    if $flags & VALID_NUM;
+  push( @flags, "VALID_STR" )    if $flags & VALID_STR;
   push( @flags, "VALID_SV" )     if $flags & VALID_SV;
   push( @flags, "REGISTER" )     if $flags & REGISTER;
   push( @flags, "TEMPORARY" )    if $flags & TEMPORARY;
@@ -149,7 +178,7 @@ sub minipeek {
   if ( $type == T_INT || $flags & VALID_INT ) {
     return $obj->{iv};
   }
-  elsif ( $type == T_DOUBLE || $flags & VALID_DOUBLE ) {
+  elsif ( $type == T_NUM || $flags & VALID_NUM ) {
     return $obj->{nv};
   }
   else {
@@ -163,28 +192,35 @@ sub minipeek {
 #
 sub set_int {
   my ( $obj, $expr, $unsigned ) = @_;
-  my $ivdformat = $Config{ivdformat};
-  $ivdformat =~ s/"//g; #" poor editor
-  my $intmax = (1 << ($Config{ivsize}*4-1)) - 1;
-  # UL if > INT32_MAX = 2147483647
-  my $sval = sprintf("%${ivdformat}%s", $expr, $expr > $intmax  ? "UL" : "");
-  if ($expr < -$intmax) {
-    $sval = sprintf("%${ivdformat}%s", $expr, "L"); # DateTime
+  my $sval;
+  # bullshit detector for non numeric expr, expr 'lnv0 + rnv0'
+  if ($expr =~ /[ a-dfzA-DF-Z]/) { # looks not like number
+    $sval = $expr;
+  } else {
+    $sval = B::C::ivx($expr);
+    $sval = $expr if $sval eq '0' and $expr;
   }
-  $sval = '0' if $sval =~ /(NAN|inf)$/i;
-  # bullshit detector: e.g. boolean expr = 'lnv0 == rnv0'
-  $sval = $expr if $sval eq '0' and $expr;
+
   runtime("$obj->{iv} = $sval;");
-  $obj->{flags} &= ~( VALID_SV | VALID_DOUBLE );
+  $obj->{flags} &= ~( VALID_SV | VALID_NUM );
   $obj->{flags} |= VALID_INT | SAVE_INT;
   $obj->{flags} |= VALID_UNSIGNED if $unsigned;
 }
 
 sub set_double {
   my ( $obj, $expr ) = @_;
-  runtime("$obj->{nv} = $expr;");
+  my $sval;
+  # bullshit detector for non numeric expr, expr 'lnv0 + rnv0'
+  if ($expr =~ /[ a-dfzA-DF-Z]/) { # looks not like number
+    $sval = $expr;
+  } else {
+    $sval = B::C::nvx($expr);
+    $sval = $expr if $sval eq '0' and $expr;
+  }
+
+  runtime("$obj->{nv} = $sval;");
   $obj->{flags} &= ~( VALID_SV | VALID_INT );
-  $obj->{flags} |= VALID_DOUBLE | SAVE_DOUBLE;
+  $obj->{flags} |= VALID_NUM | SAVE_NUM;
 }
 
 sub set_numeric {
@@ -213,19 +249,27 @@ sub set_sv {
 sub B::Stackobj::Padsv::new {
   my ( $class, $type, $extra_flags, $ix, $iname, $dname ) = @_;
   $extra_flags |= SAVE_INT    if $extra_flags & VALID_INT;
-  $extra_flags |= SAVE_DOUBLE if $extra_flags & VALID_DOUBLE;
+  $extra_flags |= SAVE_NUM if $extra_flags & VALID_NUM;
   bless {
     type  => $type,
     flags => VALID_SV | $extra_flags,
+    targ  => $ix,
     sv    => "PL_curpad[$ix]",
     iv    => "$iname",
-    nv    => "$dname"
+    nv    => "$dname",
   }, $class;
 }
 
+sub B::Stackobj::Padsv::as_obj {
+  my $obj = shift;
+  my @c = comppadlist->ARRAY;
+  my @p = $c[1]->ARRAY;
+  return $p[ $obj->{targ} ];
+}
+
 sub B::Stackobj::Padsv::load_int {
   my $obj = shift;
-  if ( $obj->{flags} & VALID_DOUBLE ) {
+  if ( $obj->{flags} & VALID_NUM ) {
     runtime("$obj->{iv} = $obj->{nv};");
   }
   else {
@@ -238,7 +282,13 @@ sub B::Stackobj::Padsv::load_double {
   my $obj = shift;
   $obj->write_back;
   runtime("$obj->{nv} = SvNV($obj->{sv});");
-  $obj->{flags} |= VALID_DOUBLE | SAVE_DOUBLE;
+  $obj->{flags} |= VALID_NUM | SAVE_NUM;
+}
+
+sub B::Stackobj::Padsv::load_str {
+  my $obj = shift;
+  $obj->write_back;
+  $obj->{flags} |= VALID_STR | SAVE_STR;
 }
 
 sub B::Stackobj::Padsv::save_int {
@@ -248,7 +298,12 @@ sub B::Stackobj::Padsv::save_int {
 
 sub B::Stackobj::Padsv::save_double {
   my $obj = shift;
-  return $obj->{flags} & SAVE_DOUBLE;
+  return $obj->{flags} & SAVE_NUM;
+}
+
+sub B::Stackobj::Padsv::save_str {
+  my $obj = shift;
+  return $obj->{flags} & SAVE_STR;
 }
 
 sub B::Stackobj::Padsv::write_back {
@@ -263,9 +318,12 @@ sub B::Stackobj::Padsv::write_back {
       runtime("sv_setiv($obj->{sv}, $obj->{iv});");
     }
   }
-  elsif ( $flags & VALID_DOUBLE ) {
+  elsif ( $flags & VALID_NUM ) {
     runtime("sv_setnv($obj->{sv}, $obj->{nv});");
   }
+  elsif ( $flags & VALID_STR ) {
+    ;
+  }
   else {
     confess "write_back failed for lexical @{[$obj->peek]}\n";
   }
@@ -282,7 +340,8 @@ sub B::Stackobj::Const::new {
   my ( $class, $sv ) = @_;
   my $obj = bless {
     flags => 0,
-    sv    => $sv    # holds the SV object until write_back happens
+    sv    => $sv,    # holds the SV object until write_back happens
+    obj   => $sv
   }, $class;
   if ( ref($sv) eq "B::SPECIAL" ) {
     $obj->{type} = T_SPECIAL;
@@ -290,7 +349,7 @@ sub B::Stackobj::Const::new {
   else {
     my $svflags = $sv->FLAGS;
     if ( $svflags & SVf_IOK ) {
-      $obj->{flags} = VALID_INT | VALID_DOUBLE;
+      $obj->{flags} = VALID_INT | VALID_NUM;
       $obj->{type}  = T_INT;
       if ( $svflags & SVf_IVisUV ) {
         $obj->{flags} |= VALID_UNSIGNED;
@@ -301,10 +360,15 @@ sub B::Stackobj::Const::new {
       }
     }
     elsif ( $svflags & SVf_NOK ) {
-      $obj->{flags} = VALID_INT | VALID_DOUBLE;
-      $obj->{type}  = T_DOUBLE;
+      $obj->{flags} = VALID_INT | VALID_NUM;
+      $obj->{type}  = T_NUM;
       $obj->{iv}    = $obj->{nv} = $sv->NV;
     }
+    elsif ( $svflags & SVf_POK ) {
+      $obj->{flags} = VALID_STR;
+      $obj->{type}  = T_STR;
+      $obj->{sv}    = $sv;
+    }
     else {
       $obj->{type} = T_UNKNOWN;
     }
@@ -317,30 +381,41 @@ sub B::Stackobj::Const::write_back {
   return if $obj->{flags} & VALID_SV;
 
   # Save the SV object and replace $obj->{sv} by its C source code name
-  $obj->{sv} = $obj->{sv}->save;
-  $obj->{flags} |= VALID_SV | VALID_INT | VALID_DOUBLE;
+  $obj->{sv} = $obj->{obj}->save;
+  $obj->{flags} |= VALID_SV | VALID_INT | VALID_NUM;
 }
 
 sub B::Stackobj::Const::load_int {
   my $obj = shift;
-  if ( ref( $obj->{sv} ) eq "B::RV" or ($] >= 5.011 and $obj->{sv}->FLAGS & SVf_ROK)) {
-    $obj->{iv} = int( $obj->{sv}->RV->PV );
+  if ( ref( $obj->{obj} ) eq "B::RV" or ($] >= 5.011 and $obj->{obj}->FLAGS & SVf_ROK)) {
+    $obj->{iv} = int( $obj->{obj}->RV->PV );
   }
   else {
-    $obj->{iv} = int( $obj->{sv}->PV );
+    $obj->{iv} = int( $obj->{obj}->PV );
   }
   $obj->{flags} |= VALID_INT;
 }
 
 sub B::Stackobj::Const::load_double {
   my $obj = shift;
-  if ( ref( $obj->{sv} ) eq "B::RV" ) {
-    $obj->{nv} = $obj->{sv}->RV->PV + 0.0;
+  if ( ref( $obj->{obj} ) eq "B::RV" ) {
+    $obj->{nv} = $obj->{obj}->RV->PV + 0.0;
   }
   else {
-    $obj->{nv} = $obj->{sv}->PV + 0.0;
+    $obj->{nv} = $obj->{obj}->PV + 0.0;
   }
-  $obj->{flags} |= VALID_DOUBLE;
+  $obj->{flags} |= VALID_NUM;
+}
+
+sub B::Stackobj::Const::load_str {
+  my $obj = shift;
+  if ( ref( $obj->{obj} ) eq "B::RV" ) {
+    $obj->{sv} = $obj->{obj}->RV;
+  }
+  else {
+    $obj->{sv} = $obj->{obj};
+  }
+  $obj->{flags} |= VALID_STR;
 }
 
 sub B::Stackobj::Const::invalidate { }
@@ -348,17 +423,17 @@ sub B::Stackobj::Const::invalidate { }
 #
 # Stackobj::Bool
 #
-
+;
 @B::Stackobj::Bool::ISA = 'B::Stackobj';
 
 sub B::Stackobj::Bool::new {
   my ( $class, $preg ) = @_;
   my $obj = bless {
     type  => T_INT,
-    flags => VALID_INT | VALID_DOUBLE,
+    flags => VALID_INT | VALID_NUM,
     iv    => $$preg,
     nv    => $$preg,
-    preg  => $preg                       # this holds our ref to the pseudo-reg
+    obj   => $preg                       # this holds our ref to the pseudo-reg
   }, $class;
   return $obj;
 }
@@ -374,6 +449,41 @@ sub B::Stackobj::Bool::write_back {
 
 sub B::Stackobj::Bool::invalidate { }
 
+#
+# Stackobj::Aelem
+#
+
+@B::Stackobj::Aelem::ISA = 'B::Stackobj';
+
+sub B::Stackobj::Aelem::new {
+  my ( $class, $av, $ix, $lvalue ) = @_;
+  my $sv;
+  # pop ix before av
+  if ($av eq 'POPs' and $ix eq 'POPi') {
+    $sv = "({ int _ix = POPi; _ix >= 0 ? AvARRAY(POPs)[_ix] : *av_fetch((AV*)POPs, _ix, $lvalue); })";
+  } elsif ($ix =~ /^-?[\d\.]+$/) {
+    $sv = "AvARRAY($av)[$ix]";
+  } else {
+    $sv = "($ix >= 0 ? AvARRAY($av)[$ix] : *av_fetch((AV*)$av, $ix, $lvalue))";
+  }
+  my $obj = bless {
+    type  => T_UNKNOWN,
+    flags => VALID_INT | VALID_NUM | VALID_SV,
+    iv    => "SvIVX($sv)",
+    nv    => "SvNVX($sv)",
+    sv    => "$sv",
+    lvalue => $lvalue,
+  }, $class;
+  return $obj;
+}
+
+sub B::Stackobj::Aelem::write_back {
+  my $obj = shift;
+  $obj->{flags} |= VALID_SV | VALID_INT | VALID_NUM | VALID_STR;
+}
+
+sub B::Stackobj::Aelem::invalidate { }
+
 1;
 
 __END__
@@ -401,12 +511,12 @@ variables according to a magic naming scheme in L<B::CC/load_pad>.
 
 Future ideas are B<type qualifiers> as attributes
 
-  B<double>, B<int>, B<register>, B<temp>, B<unsigned>, B<ro>
+  B<num>, B<int>, B<register>, B<temp>, B<unsigned>, B<ro>
 
 such as in
 
 	our int $i : unsigned : ro;
-        our double $d;
+        our num $d;
 
 Type attributes for sub definitions are not spec'ed yet.
 L<Ctypes> attributes and objects should also be recognized, such as
@@ -416,14 +526,16 @@ B<my vs our>: Note that only B<our> attributes are resolved at B<compile-time>,
 B<my> attributes are resolved at B<run-time>. So the compiler will only see
 type attributes for our variables.
 
-See L<B::CC/load_pad> and L<Ctypes>.
+See L<B::CC/load_pad> and L<types>.
 
-TODO: To represent on this stack not only PADs,SV,IV,PV,NV,BOOL,Special
-and a SV const, but also GV,CV,RV,AV,HV use B::Stackobj::Const.
+TODO: Represent on this stack not only PADs,SV,IV,PV,NV,BOOL,Special
+and a SV const, but also GV,CV,RV,AV,HV, esp. AELEM and HELEM.
+Use B::Stackobj::Const.
 
 =head1 AUTHOR
 
-Malcolm Beattie C<MICB at cpan.org> I<(retired)>
+Malcolm Beattie C<MICB at cpan.org> I<(retired)>,
+Reini Urban C<rurban at cpan.org>
 
 =cut
 
